package ar.hipotecario.multichannel.workflow;

import junit.framework.TestCase;

import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.db.GraphSession;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.graph.exe.Token;
import org.jbpm.persistence.db.DbPersistenceServiceFactory;
import org.jbpm.svc.Services;

/**
 * @author JRamos
 *
 */
public class TimerProcessTest extends TestCase {
	private final static String PROCESS_NAME = "testTimerProcess";
	private JbpmConfiguration jbpmConfiguration = null;
	private JbpmContext jbpmContext;
	private DbPersistenceServiceFactory dbPersistenceServiceFactory = null;

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#setUp()
	 */
	public void setUp() {
		jbpmConfiguration = JbpmConfiguration.parseResource("jbpm.cfg.xml");
		dbPersistenceServiceFactory = (DbPersistenceServiceFactory) jbpmConfiguration.getServiceFactory(Services.SERVICENAME_PERSISTENCE);
		dbPersistenceServiceFactory.createSchema();
		deployProcess();
		jbpmContext = jbpmConfiguration.createJbpmContext();
		jbpmConfiguration.startJobExecutor();
	}

	/**
	 * 
	 */
	public void deployProcess() {
		jbpmContext = jbpmConfiguration.createJbpmContext();
		
		try {
			ProcessDefinition processDefinition = ProcessDefinition.parseXmlResource("testTimerProcess/processdefinition.xml");
			jbpmContext.deployProcessDefinition(processDefinition);
		} finally {
			jbpmContext.close();
		}
	}
	
	/* (non-Javadoc)
	 * @see junit.framework.TestCase#tearDown()
	 */
	public void tearDown() {
		if (jbpmContext!= null && jbpmContext.getSession() != null && jbpmContext.getSession().isOpen()) jbpmContext.close();
		dbPersistenceServiceFactory.dropSchema();
		jbpmContext = null;
	}

	/**
	 * @throws InterruptedException
	 */
	public void testProcessWithTimeoutDocumentation() throws InterruptedException {
		try {
			jbpmContext = jbpmConfiguration.createJbpmContext();
			GraphSession graphSession = jbpmContext.getGraphSession();
			ProcessDefinition processDefinition = graphSession.findLatestProcessDefinition(PROCESS_NAME);
			ProcessInstance processInstance = new ProcessInstance(processDefinition);
			long idProcess = processInstance.getId();
			System.out.println("IdProceso: " + idProcess);
			Token token = processInstance.getRootToken();
			token.signal();
			assertEquals("waitUserDoc", token.getNode().getName());
			jbpmContext.save(processInstance);
			jbpmContext.close();
			
			// Wait timer trigger.
			Thread.sleep(10000);
			
			jbpmContext = jbpmConfiguration.createJbpmContext();
			jbpmConfiguration.startJobExecutor();
			graphSession = jbpmContext.getGraphSession();
			processInstance = graphSession.loadProcessInstance(idProcess);
			token = processInstance.getRootToken();
			assertEquals("notReceived", token.getNode().getName());	
			token.signal();
			assertEquals("endState", token.getNode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
	}
}
